#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['bkflow_dmn']

package_data = \
{'': ['*']}

install_requires = \
['bkflow-feel >= 1.0.0,<2', 'pydantic <3']

extras_require = \
{'test': ['pytest >=7.0.1,<8', 'pytest-benchmark[histogram] >=3.4.1,<4']}

setup(name='bkflow-dmn',
      version='0.1.1',
      description='A DMN (Decision Model and Notation) Implementation for Python',
      author=None,
      author_email='normal-wls <weishi.swee@qq.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )
